//$Id: julian_date.cxx,v 2.8 1999/11/11 19:11:15 semar Exp $
#include "julian_date.h"
#include <math.h>			/* Standard C library	*/
#include <stdlib.h>
#include <stdio.h>
#define NEWLINE 10
void JD::set_julian_date(int year,int month,int day,int hours,int minutes,
#ifdef USE_LONG_DOUBLE
		       long double seconds)
#else
		       double seconds)
#endif
{
//
//  C++ wrapper:  Cary L. Semar
//                McDonnell Douglas Aerospace Space & Defense Systems
//                July, 1996.
//  modified   :  Scott P. Cryan
//                McDonnell Douglas Aerospace Space & Defense Systems
//                September, 1995.
//  programmer :  Michael F. Fraietta
//                McDonnell Douglas Space Systems Company
//                Houston Division. January 1993.
//  comments by:  Victor R. Bond
//                McDonnell Douglas Space Systems Company
//                Houston Division. January 1993.
//
// The following formula for "jd_noon" is from the reference:
//
//     Van Flandern,T. and Pulkkinen,K.:LOW-PRECISION FORMULAE FOR
//   PLANETARY POSITIONS.Astrophysical Journal Supplement Series,
//   vol.41,pp391-411,1979 November.Also reprinted by William Bell,
//   Inc. Richmond,Virginia.
//
//   IMPORTANT  !!!!
//   On 11 Jan.1993 Van Flandern sent following  to Mike Fraietta:
//       "That version(i.e.,the formula) for jd_noon is valid for all
//        A.D.dates on the Gregorian calendar."
//
//  For proper use of this formula we quote directly :
//    " In order to convert any Gregorian calendar date into a
//      Julian date,a simple one-card FORTRAN statement algorithm
//      for the Julian day number at Greenwich mean noon is
// 
#ifdef USE_LONG_DOUBLE
        long double jd_noon = (long)(367*year) -
#else
        double jd_noon = (long)(367*year) -
#endif
                 7*(year+(month+9)/12)/4 -
                 3*((year+(month-9)/7)/100 + 1)/4 +
                 275*month/9 + day + 1721029;

//
//  The next formula shifts "jd_noon" back to midnight by subtracting
//  half a day(-0.5)then further corrects by adding the time (in days,
//  that is, hours/24 + minutes/1440 + seconds/86400.0) elapsed since
//  midnight.
// 
       julian_day = jd_noon - 0.50 +
              hours/24.0 +
              minutes/1440.0 +
              seconds/86400.0;
 
        jd_fraction = julian_day - jd_noon;

        jd_midnight = jd_noon - 0.5;

//printf("JD: %5.5f\t JDF: %5.5f\tJDN: %5.5f\n\n", julian_day, jd_fraction, jd_noon);


}

#ifdef USE_LONG_DOUBLE
long double
JD::dt_comp (long double jd)
#else
double
JD::dt_comp (double jd)
#endif
{

/*
 *	This subroutine computes the difference going from 
 *	Terrestrial Time (previously known as Terrestrial
 *	Dynamic Time (TDT) and before that, Ephemeris
 *	Time (ET)) to Universal Time (UTC), in seconds.
 *
 *	Written by Mike Lisano, MDSSC, July 1995.
 *
 *	Modifeid by Scott P. Cryan, MDSSC, September 1995.
 *
 --- For latest leap second updates, check this ftp site:
          ftp://maia.usno.navy.mil/ser7/leapsec.dat 
 *
 */

#ifdef USE_LONG_DOUBLE
long double	mjd, dettai, dtaiutc;
long double	detut;
#else
double	mjd, dettai, dtaiutc;
double	detut;
#endif
int i;
static int leap_seconds = 0;
static double julian_days[MAXLEAPSECONDS];
static double deltas[MAXLEAPSECONDS];
static int table_loaded = 0;
dettai = 32.1840;

mjd = jd - 2400000.50;

if (jd < 2437300.50) dtaiutc = 0.0;

if (jd >= 2437300.50 && jd < 2437512.50)
	dtaiutc = 1.42281800 + (mjd - 37300.0) * 0.0012960;

 if (jd >= 2437512.50 && jd < 2437665.50)
	dtaiutc = 1.37281800 + (mjd - 37300.0) * 0.0012960;

 if (jd >= 2437665.50 && jd < 2438334.50)
	dtaiutc = 1.84585800 + (mjd - 37665.0) * 0.00112320;

 if (jd >= 2438334.50 && jd < 2438395.50)
	dtaiutc = 1.94585800 + (mjd - 37665.0) * 0.00112320;

 if (jd >= 2438395.50 && jd < 2438486.50)
	dtaiutc = 3.24013000 + (mjd - 38761.0) * 0.0012960;

 if (jd >= 2438486.50 && jd < 2438639.50)
	dtaiutc = 3.34013000 + (mjd - 38761.0) * 0.0012960;

 if (jd >= 2438639.50 && jd < 2438761.50)
	dtaiutc = 3.44013000 + (mjd - 38761.0) * 0.0012960;

 if (jd >= 2438761.50 && jd < 2438820.50)
	dtaiutc = 3.54013000 + (mjd - 38761.0) * 0.0012960;

 if (jd >= 2438820.50 && jd < 2438942.50)
	dtaiutc = 3.64013000 + (mjd - 38761.0) * 0.0012960;

 if (jd >= 2438942.50 && jd < 2439004.50)
	dtaiutc = 3.74013000 + (mjd - 38761.0) * 0.0012960;

 if (jd >= 2439004.50 && jd < 2439126.50)
	dtaiutc = 3.84013000 + (mjd - 38761.0) * 0.0012960;

 if (jd >= 2439126.50 && jd < 2439887.50)
	dtaiutc = 4.31317000 + (mjd - 39126.0) * 0.0025920;

 if (jd >= 2439887.50 && jd < 2441317.50)
	dtaiutc = 4.21317000 + (mjd - 39126.0) * 0.0025920;

 if (jd >= 2441317.50 && jd < 2441499.50)
	dtaiutc = 10.0;

 if (jd >= 2441499.50 && jd < 2441683.50)
	dtaiutc = 11.0;

 if (jd >= 2441683.50 && jd < 2442048.50)
	dtaiutc = 12.0;

 if (jd >= 2442048.50 && jd < 2442413.50)
	dtaiutc = 13.0;

 if (jd >= 2442413.50 && jd < 2442778.50)
	dtaiutc = 14.0;

 if (jd >= 2442778.50 && jd < 2443144.50)
	dtaiutc = 15.0;

 if (jd >= 2443144.50 && jd < 2443509.50)
	dtaiutc = 16.0;

 if (jd >= 2443509.50 && jd < 2443874.50)
	dtaiutc = 17.0;

 if (jd >= 2443874.50 && jd < 2444239.50)
	dtaiutc = 18.0;
 if (jd >= 2444239.50 && jd < 2444786.50)
	dtaiutc = 19.0;

 if (jd >= 2444786.50 && jd < 2445151.50)
	dtaiutc = 20.0;

 if (jd >= 2445151.50 && jd < 2445516.50)
	dtaiutc = 21.0;

 if (jd >= 2445516.50 && jd < 2446247.50)
	dtaiutc = 22.0;

 if (jd >= 2446247.50 && jd < 2447161.50)
	dtaiutc = 23.0;

 if (jd >= 2447161.50 && jd < 2447892.50)
	dtaiutc = 24.0;

 if (jd >= 2447892.50 && jd < 2448257.50)
	dtaiutc = 25.0;

 if (jd >= 2448257.50 && jd < 2448804.50)
	dtaiutc = 26.0;

 if (jd >= 2448804.50 && jd < 2449169.50)
	dtaiutc = 27.0;

 if (jd >= 2449169.50 && jd < 2449534.50)
	dtaiutc = 28.0;

 if (jd >= 2449534.50 && jd < 2450083.50)
	dtaiutc = 29.0;

 if (jd >= 2450083.50)
	dtaiutc = 30.0;

 if (jd >= 2450630.50)  
	dtaiutc = 31.0;

 if (jd >= 2451179.50)  //--- Jan 1, 1999 leap second change
	dtaiutc = 32.0;
/*
 *  THE ABOVE TABLES ARE BASED ON THE IERS BULLETIN (July 27 1995),
 *  WITH THE FINAL LINEAR INTERPOLATION IN DTAIUTC BEING Mike Lisano's
 *  APPROXIMATION OF THE 1995-2005 IERS TT-UT PREDICTION... 
 */
 if(!leap_seconds && !table_loaded)
 {
    leap_seconds = load_leapseconds(julian_days,deltas);
    table_loaded = 1;
 }
 if(leap_seconds>1)
 {
    for(i=0;i<leap_seconds;++i)
    {
      if(jd >= julian_days[i]) dtaiutc = deltas[i];
    } 
 
 }
 detut = dettai + dtaiutc;
// printf("dtaiutc = %f detut = %f\n",dtaiutc,detut);
 return (detut);
}
int JD::load_leapseconds(double julian_days[],double deltas[])
{
 FILE* config_file;
 char sy[80];
 char c;
 int ncount;
 double jd_table,dta_table;
 ncount = 0;
 config_file = fopen("leapsec.dat","r");
 if(!config_file)
 {
   
   printf("The leap seconds table was not found.\n");
   printf("The leap seconds table  is the file which contains the dates \n");
   printf("for UTC leap seconds needed for conversion of UTC to GPS time.\n");
   printf("The default table extend to 1999/01/01.\n");
   printf("Type return to continue.\n");
   gets(sy);
   return 0;
 }

 int npar;
 while(!feof(config_file))
 { 
   npar = fscanf(config_file,"%s %s %s %s %lf %s %lf",
         sy,sy,sy,sy,&jd_table,sy,&dta_table);
   if(feof(config_file)) break;
   if(npar != 7)
   {
     printf("Error in leap seconds table.\n");
     break;
   }
   while(fgetc(config_file) != NEWLINE);
// printf(" %f %f\n",jd_table,dta_table);
   julian_days[ncount] = jd_table;
   deltas[ncount] = dta_table;
   if(ncount < MAXLEAPSECONDS)
      ++ncount;
   else printf("Table too long. Reduce table to %d entries.\n",MAXLEAPSECONDS);
 }
 fclose(config_file);
//printf("ncount = %d\n",ncount);
 return ncount;
}



void	
#ifdef USE_LONG_DOUBLE
JD::calendar_date(long double julian_day, 
	     long double jd_fraction,
	     int &yr, int &mo,
	     int &dy, int &hr,
	     int &mn, long double &sc)
#else
JD::calendar_date(double julian_day,
             double jd_fraction,
             int &yr, int &mo,
             int &dy, int &hr,
             int &mn, double &sc)
#endif
{
    
#ifdef USE_LONG_DOUBLE
  long double temp1, temp2;
  long double jd;
#else
  double temp1, temp2;
  double jd;
#endif
  int	lx, nx;

  temp1    = jd_fraction*24.0 + 12.0;
  hr  = (int)temp1;

  temp2	 = (temp1 - hr) * 60.0;
  mn = (int)temp2;
//  if( (temp2 - (int)temp2) < 0.5)
//  { mn = (int)floor(temp2); }
//  else
//  { mn = (unsigned int)ceil(temp2); }

  sc = (temp2 - mn) * 60.0;

//  if (sc >= 60.0)
  if (sc >= 59.999999999)
    {
      sc = 0.0;
      mn++;
    }
  if (mn >= 60)
    {
      mn = 0;
      hr++;
    }
  if (hr >= 24)
    hr = 0;
  

  if (hr < 12)
    jd = julian_day +1;
  else
    jd = julian_day;


//  printf("CALENDAR>>>\n%d/%d/%d %d:%d:%5.2f\n\n",
//	 yr,mo,dy,hr,mn,sc);

  
  lx = (int)jd + 68569;
  nx = 4 * lx / 146097;
  lx = lx - (146097 * nx + 3) / 4;
  yr = 4000 * (lx + 1)/1461001;
  lx = lx - 1461 * yr/4 + 31;
  mo = 80 * lx / 2447;
  dy = lx - 2447 * mo/80;
  lx = mo / 11;
  mo = mo + 2 - (12 * lx);
  yr = 100 * (nx-49) + yr + lx;
  

  return;
}
