//$Id: time.cxx,v 2.3 1999/04/08 19:45:22 pinon Exp pinon $
#include "time.h"

UTC  Time::utc_launch_t;
GPST Time::gps_launch_t;


////////////////////////////////////
//
// Set the time based on UTC
//
////////////////////////////////////

void 
Time::set(UTC launch_t)
{
  utc_launch_t = launch_t;

  convert(&utc_launch_t, &gps_launch_t);

  return;
}


////////////////////////////////////
//
// Set the time based on Y/M/D H:M:S
//
////////////////////////////////////

void 
Time::set(int yr, int mo,
	  int dy, int hr,
#ifdef USE_LONG_DOUBLE
	  int mn, long double sc)
#else
	  int mn, double sc)
#endif
{
  utc_launch_t.set(yr, mo, dy, hr, mn, sc);
  
  set_julian_date(yr, mo, dy, hr, mn, sc);
  
  convert(&utc_launch_t, &gps_launch_t);

  return;
}
  

////////////////////////////////////
//
// Get the time based on UTC
//
////////////////////////////////////

void 
Time::get(UTC *launch_t)
{ 
  int    yr,mo,dy,hr,mn;
#ifdef USE_LONG_DOUBLE
  long double sc;
#else
  double sc;
#endif
  
  utc_launch_t.get(&yr,&mo,&dy,&hr,&mn,&sc);
  launch_t->set(yr,mo,dy,hr,mn,sc);

  return;
} 
 

////////////////////////////////////
//
// Get the time based on Y/M/D H:M:S
//
////////////////////////////////////

void 
Time::get(int *year, int *month,
	  int *day,  int *hour,
#ifdef USE_LONG_DOUBLE
	  int *min,  long double *sec)
#else
	  int *min,  double *sec)
#endif
{
  int    yr,mo,dy,hr,mn;
#ifdef USE_LONG_DOUBLE
  long double sc;
#else
  double sc;
#endif
  
  utc_launch_t.get(&yr,&mo,&dy,&hr,&mn,&sc);
 
  *year  = yr;   
  *month = mo;    
  *day   = dy;
  *hour  = hr;
  *min   = mn;
  *sec   = sc;

  return;
}

////////////////////////////////////
//
// 
//
////////////////////////////////////

void
Time::print(void)
{
  utc_launch_t.print();
  gps_launch_t.print();
}

////////////////////////////////////
//
//  Convert from UTC to GPS time
//
////////////////////////////////////

void
Time::convert(const UTC  *utc, GPST *gps)
{
#ifdef USE_LONG_DOUBLE
  long double days, detut, sec;
#else
  double days, detut, sec;
#endif
  int week;


  //
  // Calclulate lag times 
  //
  detut = dt_comp (utc->julian_day);

  //
  // Number of days since gps epoch
  //
  days = utc->julian_day - JULIAN_DAY_REF;

  //
  // Convert numbers of days into wk/sc
  //
  week = (int)(days / 7.0);

  sec =   (days - (week*7.0)) * T_FULL_DAY +
                  (detut - DETUT_REF);

  //
  // Set the values in the gps class
  //
  gps->set( week, sec);
  
  return;
}
 


////////////////////////////////////
//
//  Convert from MET to GPS time
//
////////////////////////////////////

void
Time::convert(const MET *met, GPST *gps)
{

  UTC utc;

  convert(met, &utc);
  convert(&utc, gps);

  return;
}



////////////////////////////////////
//
//  Convert from Shuttle time 
//  to GPS time
//
////////////////////////////////////
 
void
Time::convert(const STST *st, GPST *gps)
{

  UTC utc;

  convert(st, &utc);
  convert(&utc, gps);

  return;
} 



////////////////////////////////////
//
//  Convert from GPS_Clock time 
//  to GPS time
//
////////////////////////////////////
 
void
Time::convert(const GPSclock *gpsc, GPST *gpst)
{
#ifdef USE_LONG_DOUBLE
  long double days, sec;
#else
  double days, sec;
#endif
  int week;


  //
  // Subtract the gpsc jd from gps epoch
  //
  days = gpsc->julian_day - JULIAN_DAY_REF;


  //
  // Set Values
  //
  week = (int)(days / 7.0);
  sec =   (days - (week*7.0)) * T_FULL_DAY;
  
  gpst->set( week, sec);
  

  return;

} 

////////////////////////////////////
////////////////////////////////////
////////////////////////////////////
////////////////////////////////////







////////////////////////////////////
//
//  Convert from GPS to UTC time
//
////////////////////////////////////

void
Time::convert(const GPST *gps, UTC *utc)
{
#ifdef USE_LONG_DOUBLE
  long double detut, jd, JDfraction;
  long double sc;
#else
  double detut, jd, JDfraction;
  double sc;
#endif

  int	 JDinteger;
  int    yr,mo,dy,hr,mn;
  
  detut	= dt_comp (gps->julian_day);
  jd    = gps->julian_day - (detut - DETUT_REF)/T_FULL_DAY;
  
  JDinteger      = (int)jd;
  JDfraction = jd - JDinteger;

  if (JDfraction > 0.5000)
    JDfraction -= 1.00000;
  
  calendar_date (jd, JDfraction,
		 yr,  mo,  dy, 
		 hr,  mn,  sc);
  
  utc->set(yr, mo, dy, hr, mn, sc);

  return;

}
 


////////////////////////////////////
//
//  Convert from MET to UTC time
//
////////////////////////////////////

void
Time::convert(const MET *met, UTC *utc)
{
#ifdef USE_LONG_DOUBLE
  long double JD, JD_fraction;
  long double sc;
#else
  double JD, JD_fraction;
  double sc;
#endif

  int    JD_integer;
  int    yr,mo,dy,hr,mn;


#ifdef USE_LONG_DOUBLE
  printf("tjd:  %Lf  mjd  %Lf\n", utc_launch_t.julian_day, met->julian_day);
#else
  printf("tjd:  %lf  mjd  %lf\n", utc_launch_t.julian_day, met->julian_day);
#endif
  //
  // Sum the JD values for the time of launch
  // and the elapse time since launch.  Sub a
  // half day to synch with Greenwich time
  //
  JD = utc_launch_t.julian_day + met->julian_day - 0.50;

  JD_integer = (int)(JD + 0.50);

#ifdef USE_LONG_DOUBLE
  JD_fraction = (JD - (long double)JD_integer);
#else
  JD_fraction = (JD - (double)JD_integer);
#endif


  //
  // Calculate the calandar date from the JD 
  // values and write them into the yr/mo/dy
  // values.  
  //
  calendar_date(JD, JD_fraction, 
		yr,  mo,  dy, 
		hr,  mn,  sc);
  

  //
  // Set the returned values from cal_date
  // in the 'utc' instance.
  //
  utc->set(yr, mo, dy, hr, mn, sc);

  return;

} 



////////////////////////////////////
//
//  Convert from STST to UTC time
//
////////////////////////////////////

void
Time::convert(const STST *st, UTC *utc)
{
#ifdef USE_LONG_DOUBLE
  long double JD, JD_fraction;
  long double sc;
#else
  double JD, JD_fraction;
  double sc;
#endif

  int    JD_integer;
  int    yr,mo,dy,hr,mn;


  //
  // Sum the JD values for the time of launch
  // and the elapse time since launch.  Sub a
  // half day to synch with Greenwich time
  //
  JD = st->julian_day;

  JD_integer = (int)(JD + 0.50);

#ifdef USE_LONG_DOUBLE
  JD_fraction = (JD - (long double)JD_integer);
#else
  JD_fraction = (JD - (double)JD_integer);
#endif


  //
  // Calculate the calandar date from the JD 
  // values and write them into the yr/mo/dy
  // values.  
  //
  calendar_date(JD, JD_fraction, 
		yr,  mo,  dy, 
		hr,  mn,  sc);
  

  //
  // Set the returned values from cal_date
  // in the 'utc' instance.
  //
  utc->set(yr, mo, dy, hr, mn, sc);

} 


////////////////////////////////////
//
//  Convert from GPSclock to UTC time
//
////////////////////////////////////

void
Time::convert(const GPSclock *gpsc, UTC *utc)
{

  GPST gps;

  convert(gpsc, &gps);
  convert(&gps, utc);

  return;
} 


////////////////////////////////////
////////////////////////////////////
////////////////////////////////////
////////////////////////////////////


////////////////////////////////////
//
//  Convert from GPST to MET time
//
////////////////////////////////////

void
Time::convert(const GPST *gps, MET *met)
{

#ifdef USE_LONG_DOUBLE
  long double JD, sc;
#else
  double JD, sc;
#endif
  UTC    utc;
  

  //
  // JD in UTC is needed to determine
  // value for MET.
  //
  convert(gps, &utc);


  //
  // Difference is MET JD.  Multiply by 
  // number of seconds in dat to get MET.
  //
  sc = (utc.julian_day - utc_launch_t.julian_day)*T_FULL_DAY;  


  //
  // Set the value
  //
  met->set(sc);

  return;

}


////////////////////////////////////
//
//  Convert from UTC to MET time
//
////////////////////////////////////

void 
Time::convert(const UTC *utc, MET *met)
{
#ifdef USE_LONG_DOUBLE
  long double sc;
#else
  double sc;
#endif

  //
  // Difference is MET JD.  Multiply by 
  // number of seconds in dat to get MET.
  //
  sc = (utc->julian_day - utc_launch_t.julian_day)*T_FULL_DAY;  


  //
  // Set the value
  //
  met->set(sc);

  return;

} 

////////////////////////////////////
//
//  Convert from GPS to MET time
//
////////////////////////////////////

void 
Time::convert(const STST *st, MET *met)
{

  UTC utc;

  convert(st, &utc);
  convert(&utc, met);

  return;
} 

////////////////////////////////////
//
//  Convert from GPS to MET time
//
////////////////////////////////////

void 
Time::convert(const GPSclock *gpsc, MET *met)
{

  GPST gps;

  convert(gpsc, &gps);
  convert(&gps, met);
  
  return;
} 

////////////////////////////////////
////////////////////////////////////
////////////////////////////////////
////////////////////////////////////


////////////////////////////////////
//
//  Convert from GPS time 
//  to Shuttle time
//
////////////////////////////////////
 
void
Time::convert(const GPST *gps, STST *st)
{

  UTC utcTime, utcYear;
  int    yr,mo,dy,hr,mn;
#ifdef USE_LONG_DOUBLE
  long double sc, jd;
#else
  double sc, jd;
#endif

  //
  //  Need to find the UTC year
  //
  convert(gps, &utcTime);
  utcTime.get(&yr,&mo,&dy,&hr,&mn,&sc);

  //
  // Set utcYear to get JD for the 
  // beginning of that year
  //
  utcYear.set(yr-1,12,31,0,0,0.0); 

  //
  // Difference of utcTime's and utcYear's
  // JD is the number of JDs in the current
  // year.
  //
  jd = utcTime.julian_day - utcYear.julian_day;

  //
  // Convert the jd to seconds.
  //
  sc = jd*T_FULL_DAY;
  
  
  //
  // Set the STST value.
  //
  st->set(yr, sc);

  return;
} 


////////////////////////////////////
//
//  Convert from UTC time 
//  to Shuttle time
//
////////////////////////////////////

void 
Time::convert(const UTC *utc, STST *st)
{

  GPST gps;

  convert(utc, &gps);
  convert(&gps, st);

  return;
} 


////////////////////////////////////
//
//  Convert from MET time 
//  to Shuttle time
//
////////////////////////////////////

void 
Time::convert(const MET *met, STST *st)
{

  GPST gps;

  convert(met, &gps);
  convert(&gps, st);
  
  return;
} 


////////////////////////////////////
//
//  Convert from GPSclock time 
//  to Shuttle time
//
////////////////////////////////////

void 
Time::convert(const GPSclock *gpsc, STST *st)

{

  GPST gps;

  convert(gpsc, &gps);
  convert(&gps, st);

  return;
} 

////////////////////////////////////
////////////////////////////////////
////////////////////////////////////
////////////////////////////////////



////////////////////////////////////
//
//  Convert from GPS time 
//  to GPSclock time
//
////////////////////////////////////
 
void
Time::convert(const GPST *gpst, GPSclock *gpsc)
{
#ifdef USE_LONG_DOUBLE
  long double        JDfraction;
  long double sc;
#else
  double        JDfraction;
  double sc;
#endif
  int		JDinteger;
  int    yr,mo,dy,hr,mn;
  
  JDinteger      = (int)gpst->julian_day;
  JDfraction = gpst->julian_day - JDinteger;

  if (JDfraction > 0.5000)
    JDfraction -= 1.00000;
  
  calendar_date (gpst->julian_day, JDfraction,
		 yr,  mo,  dy, 
		 hr,  mn,  sc);
  
  gpsc->set(yr, mo, dy, hr, mn, sc);

  return;

} 


////////////////////////////////////
//
//  Convert from UTC time 
//  to GPSclock time
//
////////////////////////////////////

void 
Time::convert(const UTC *utc, GPSclock *gpsc)
{

  GPST gps;

  convert(utc, &gps);
  convert(&gps, gpsc);

  return;
} 

////////////////////////////////////
//
//  Convert from MET time 
//  to GPSclock time
//
////////////////////////////////////

void 
Time::convert(const MET *met, GPSclock *gpsc)
{

  GPST gps;

  convert(met, &gps);
  convert(&gps, gpsc);

  return;
} 

////////////////////////////////////
//
//  Convert from STST time 
//  to GPSclock time
//
////////////////////////////////////

void 
Time::convert(const STST *st, GPSclock *gpsc)
{

  GPST gps;

  convert(st, &gps);
  convert(&gps, gpsc);

  return;
} 
